/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.hotmail.frojasg1.applications.fileencoder.fileencodertypes;

import com.frojasg1.general.GeneralException;
import com.frojasg1.general.HexadecimalFunctions;
import com.frojasg1.general.StringFunctions;
import com.hotmail.frojasg1.applications.fileencoder.FileEncoder;
import com.hotmail.frojasg1.applications.fileencoder.FileEncoderException;
import com.hotmail.frojasg1.applications.fileencoder.FileEncoderParameters;
import com.hotmail.frojasg1.encrypting.encoderdecoder.EncoderDecoder;
import com.hotmail.frojasg1.encrypting.encoderdecoder.ReordererEncoder;
import com.frojasg1.general.progress.CancellationException;
import com.frojasg1.general.progress.OperationCancellation;
import com.frojasg1.general.progress.UpdatingProgress;
import com.hotmail.frojasg1.encrypting.pseudorandomgenerator.PseudoRandomGenerator;
import com.hotmail.frojasg1.encrypting.pseudorandomgenerator.PseudoRandomGeneratorException;
import com.hotmail.frojasg1.encrypting.pseudorandomgenerator.chaoticgenerator.ChaoticGenerator_BigLong_depretated;
import com.hotmail.frojasg1.encrypting.randomnumbers.RandomSource;
import java.io.File;
import java.nio.file.Files;
import java.util.Arrays;

/**
 *
 * @author axpe
 */
@Deprecated
public class FileEncoderType_3 extends FileEncoderType_2
{
	public FileEncoderType_3( UpdatingProgress up )
	{
		super( up );
	}

	public int M_getFileEncoderType()
	{
		return( FileEncoderParameters.INT_FILE_ENCODER_TYPE_3 );
	}

	protected byte getByteValueOfSize( int size ) throws FileEncoderException
	{
		if( ( size < 1 ) || ( size > 256 ) )
			throw( new FileEncoderException( "Error, size of number out of range: " + size ) );

		byte result = (byte) size;
		
		if( size > 127 )
			result = (byte) ( size - 256 );
		
		return( result );
	}

	@Override
	protected byte[] M_encodeHeader( byte[] encodedKey, byte[] signature ) throws FileEncoderException
	{
		byte[] result = super.M_encodeHeader( encodedKey, signature );

		int headerHeadLength = a_fep.M_getHeaderHead().getBytes().length;
		result[ headerHeadLength + 1 ] = getByteValueOfSize( a_fep.M_getSizeOfNumbersSimpleEncoder() );
		result[ headerHeadLength + 2 ] = getByteValueOfSize( a_fep.M_getSizeOfNumbersReordererEncoder() );

		return( result );
	}

	protected int getUnsignedValueForSize( byte byteSizeOfNumber )
	{
		int result = byteSizeOfNumber;

		if( result < 1 )
			result = result + 256;

		return( result );
	}

	@Override
	protected void M_updateParametersWithHeader( byte[] header, String fileName ) throws FileEncoderException
	{
		super.M_updateParametersWithHeader( header, fileName );

		byte sizeOfNumbersSimpleEncoder = (byte) a_fep.M_getSizeOfNumbersSimpleEncoder();
		byte sizeOfNumbersReordererEncoder = (byte) a_fep.M_getSizeOfNumbersReordererEncoder();
		
		a_fep.M_setSizeOfNumbersSimpleEncoder( getUnsignedValueForSize( sizeOfNumbersSimpleEncoder ) );
		a_fep.M_setSizeOfNumbersReordererEncoder( getUnsignedValueForSize( sizeOfNumbersReordererEncoder ) );
	}

	@Override
	public void M_checkSizeOfNumbers( FileEncoderParameters fep ) throws FileEncoderException
	{
		if( ( fep.M_getSizeOfNumbersSimpleEncoder() < 1 ) || (fep.M_getSizeOfNumbersSimpleEncoder() > 256 ) )
			throw( new FileEncoderException( "Bad value for SizeOfNumbersSimpleEncoder. It must be between 1 and 256" ) );
		
		if( ( fep.M_getSizeOfNumbersReordererEncoder() < 1 ) || (fep.M_getSizeOfNumbersReordererEncoder() > 256 ) )
			throw( new FileEncoderException( "Bad value for SizeOfNumbersReordererEncoder. It must be between 1 and 256" ) );
	}
}
