/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general;

import java.util.Iterator;
import java.util.List;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ArrayFunctions
{
	public static String[] joinStringArrays(String[] array1, String[] array2) {
		String[] result = null;
		if (array1 == null) {
			result = array2;
		} else if (array2 == null) {
			result = array1;
		} else {
			result = new String[array1.length + array2.length];
			System.arraycopy(array1, 0, result, 0, array1.length);
			System.arraycopy(array2, 0, result, array1.length, array2.length);
		}
		return result;
	}

	public static String[] getArrayFromList( List<String> list )
	{
		String[] result = null;

		if( list != null )
		{
			result = new String[ list.size() ];

			Iterator<String> it = list.iterator();
			for( int ii=0; it.hasNext(); ii++ )
			{
				result[ii]=it.next();
			}
		}

		return( result );
	}

	public static String[] getArrayJoiningLists( List<String> list1, List<String> list2 )
	{
		return( joinStringArrays( getArrayFromList(list1), getArrayFromList(list2 ) ) );
	}

	public static int getFirstIndexOfReference( String[] array, String reference )
	{
		int index = -1;
		
		if( array != null )
		{
			for( int ii=0; (index==-1) && (ii<array.length); ii++ )
			{
				if( array[ii] == reference )
					index = ii;
			}
		}

		return( index );
	}

	public static int getFirstIndexOfEquals( String[] array, String stringToFind )
	{
		int index = -1;
		
		if( ( array != null ) && (stringToFind != null ) )
		{
			for( int ii=0; (index==-1) && (ii<array.length); ii++ )
			{
				if( stringToFind.equals( array[ii] ) )
					index = ii;
			}
		}

		return( index );
	}
}
