/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

/**
 *
 * @author Usuario
 */
public class CollectionFunctions 
{

	public static final Comparator<Object> _ASCENDANT_NO_CASE_SENSITIVE = new Comparator<Object> () {
			@Override
			public int compare( Object one, Object two )
			{
				int result = 0;
				if( ( one == null ) && ( two == null ) )
					result = 0;
				else if( one != null )
				{
					result = 1;

					if( two != null )
					{
						if( ( one.toString() == null ) && (two.toString() == null ) )
							result = 0;
						else if( two.toString() != null )
						{
							result = -1;
							if( one.toString() != null )
								result = - two.toString().toLowerCase().compareTo( one.toString().toLowerCase() );
						}
					}
				}

				return( result );
			}
	};

	public static void sortNoCaseSensitive( List list )
	{
		Collections.sort( list, _ASCENDANT_NO_CASE_SENSITIVE );
	}

	public static int indexOfReference( Collection col, Object obj )
	{
		int result=-1;
		
		int index = 0;
		Iterator it = col.iterator();
		while( (result==-1) && it.hasNext() )
		{
			if( it.next() == obj )
				result = index;
			index++;
		}
		return( result );
	}

	public static void removeRepeatedSortedPreviously( List list )
	{
		Object[] array = list.toArray();

		for( int ii=array.length-2; ii>=0; ii-- )
		{
			if( array[ii+1].equals( array[ii] ) )
			list.remove(ii);
		}
	}

	public static boolean itemExistsInList( List list, Object item )
	{
		boolean result = false;
		
		if( list != null )
		{
			Iterator it = list.iterator();
			while( !result && it.hasNext() )
			{
				Object current = it.next();
				result = ( current == item ) || ( current != null ) && ( current.equals(item) );
			}
		}

		return( result );
	}
	
	public static List removeItemsThatExistInTheSecondList( List list, List itemsToRemove )
	{
		List removedItems = new ArrayList();
		
		Object[] array = list.toArray();
		for( int ii=array.length-1; ii>=0; ii-- )
		{
			if( itemExistsInList( itemsToRemove, array[ii] ) )
			{
				removedItems.add( list.remove(ii) );
			}
		}

		return( removedItems );
	}

	public static List leaveOnlyExistingInTheSecondList( List list, List itemsToKeep )
	{
		List result = new ArrayList();
		
		Iterator it = list.iterator();
		while( it.hasNext() )
		{
			Object current = it.next();
			if( itemExistsInList( itemsToKeep, current ) )
			{
				result.add( current );
			}
		}

		return( result );
	}
}


