/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.generic;

import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.general.desktop.dialogs.implementation.DesktopDialogsWrapper;
import com.frojasg1.general.desktop.files.DesktopFileFacilities;
import com.frojasg1.general.desktop.keyboard.DesktopKeyboardFacilities;
import com.frojasg1.general.dialogs.DialogsWrapper;
import com.frojasg1.general.files.GenericFileFacilities;
import com.frojasg1.general.keyboard.GenericKeyboard;
import com.frojasg1.generic.GenericFunctions;
import com.frojasg1.generic.GenericInterface;
import com.frojasg1.generic.languages.ObtainAvailableLanguages_base;
import com.frojasg1.generic.languages.ObtainAvailableLanguages_int;

/**
 *
 * @author Usuario
 */
public class DesktopGenericFunctions implements GenericInterface
{
	BaseApplicationConfigurationInterface _appliConf = null;
	
	protected static DesktopGenericFunctions _instance = null;

	protected ObtainAvailableLanguages_int _obtainAvailableLanguages = null;

	public static DesktopGenericFunctions createInstance( BaseApplicationConfigurationInterface conf )
	{
		_instance = new DesktopGenericFunctions( conf );
		GenericFunctions.instance().register( _instance );
		
		return( _instance );
	}

	public static DesktopGenericFunctions instance()
	{
		return( _instance );
	}

	public void setObtainAvailableLanguages( ObtainAvailableLanguages_int oal )
	{
		_obtainAvailableLanguages = oal;
	}

	public DesktopGenericFunctions( BaseApplicationConfigurationInterface conf )
	{
		_appliConf = conf;
		DesktopDialogsWrapper.createInstance( conf );

		if( ObtainAvailableLanguages_base.instance() == null )
		{
			if( conf != null )
			{
				String[] languages = new String[]{ "EN", "ES", "CAT" };
				_obtainAvailableLanguages = ObtainAvailableLanguages_base.create(languages, languages, conf.getDefaultLanguageConfigurationFolder("") );
			}
		}
		else
			_obtainAvailableLanguages = ObtainAvailableLanguages_base.instance();
	}

	@Override
	public BaseApplicationConfigurationInterface getAppliConf()
	{
		return( _appliConf );
	}

	@Override
	public DialogsWrapper getDialogsWrapper()
	{
		return( DesktopDialogsWrapper.instance() );
	}

	@Override
	public GenericFileFacilities getFileFacilities()
	{
		return( DesktopFileFacilities.instance() );
	}

	@Override
	public GenericKeyboard getKeyboardFacilities()
	{
		return( DesktopKeyboardFacilities.instance() );
	}

	@Override
	public ObtainAvailableLanguages_int getObtainAvailableLanguages()
	{
		return( _obtainAvailableLanguages );
	}
}
