/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.mouse;

import java.awt.AWTException;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Robot;

/**
 *
 * @author Usuario
 */
public class MouseFunctions
{
	
	// function got from:
	// http://stackoverflow.com/questions/2941324/how-do-i-set-the-position-of-the-mouse-in-java
	public static void moveMouse(Point p)
	{
		GraphicsEnvironment ge = 
			GraphicsEnvironment.getLocalGraphicsEnvironment();
		GraphicsDevice[] gs = ge.getScreenDevices();

		// Search the devices for the one that draws the specified point.
		for (GraphicsDevice device: gs)
		{ 
			GraphicsConfiguration[] configurations =
				device.getConfigurations();
			for (GraphicsConfiguration config: configurations)
			{
				Rectangle bounds = config.getBounds();
				if(bounds.contains(p))
				{
					// Set point to screen coordinates.
					Point b = bounds.getLocation(); 
					Point s = new Point(p.x - b.x, p.y - b.y);

					try
					{
						Robot r = new Robot(device);
						r.mouseMove(s.x, s.y);
					}
					catch (AWTException e)
					{
						e.printStackTrace();
					}

					return;
				}
			}
		}
		// Couldn't move to the point, it may be off screen.
		return;
	}

	public static Point getMouseLocation()
	{
		return( MouseInfo.getPointerInfo().getLocation() );
	}
}
