/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.license;

import com.frojasg1.general.ResourceFunctions;
import com.frojasg1.general.StringFunctions;
import com.frojasg1.general.desktop.files.FileFunctions;
import com.frojasg1.general.desktop.mouse.AvailableCursors;
import com.frojasg1.general.desktop.mouse.MouseFunctions;
import com.frojasg1.general.desktop.view.editorkits.WrapEditorKit;
import com.frojasg1.general.number.IntegerFunctions;
import com.frojasg1.general.screen.ScreenFunctions;
import com.hotmail.frojasg1.applications.common.components.resizecomp.MapResizeRelocateComponentItem;
import java.awt.Component;
import java.awt.Desktop;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.net.URI;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class SimpleRtfDocJPanel extends JPanel implements MouseListener, MouseMotionListener, MouseWheelListener
{
	protected MapResizeRelocateComponentItem _mapRRCI = null;
	protected JEditorPane _rtfEditorPane = null;

	protected String _lastURL = null;

	/**
	 * Creates new form SimpleRtfDocJPanel
	 */
	public SimpleRtfDocJPanel()
	{
		initComponents();

//		setPanelConfiguration();
	}

	public void addListeners()
	{
/*		_mapRRCI = new MapResizeRelocateComponentItem();
		try
		{
			_mapRRCI.putResizeRelocateComponentItem( jScrollPane1, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
*/
		jScrollPane1.getViewport().getView().addMouseListener( this );
		jScrollPane1.getViewport().getView().addMouseMotionListener( this );
		jScrollPane1.getViewport().getView().addMouseWheelListener(this);
	}

	public MapResizeRelocateComponentItem getMapResizeRelocateComponentItem()
	{
		return( _mapRRCI );
	}

	public void setRtfDocument( Document doc )
	{
		_rtfEditorPane = new JEditorPane( "text/rtf", "" );
		_rtfEditorPane.setEditable( false );
		_rtfEditorPane.setEditorKit( new WrapEditorKit() );
		_rtfEditorPane.setDocument( doc );
		jScrollPane1.setViewportView( _rtfEditorPane );
		_rtfEditorPane.setSelectionStart(0);
		_rtfEditorPane.setSelectionEnd(0);

		addListeners();
//		_rtfEditorPane.addMouseListener( this );
//		_rtfEditorPane.addMouseMotionListener( this );
	}

	public void setRtfFile( String fileName )
	{
		Document doc = null;
		doc = FileFunctions.loadRtfDocumentFromFile( fileName );
		setRtfDocument( doc );
	}

	public void setRtfResource( String resourceName )
	{
		Document doc = null;
		doc = ResourceFunctions.loadResourceRTFDocument( resourceName );
		setRtfDocument( doc );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        jEditorPane1 = new javax.swing.JEditorPane();

        setMinimumSize(new java.awt.Dimension(50, 50));
        setLayout(new java.awt.BorderLayout());

        jScrollPane1.setViewportView(jEditorPane1);

        add(jScrollPane1, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JEditorPane jEditorPane1;
    private javax.swing.JScrollPane jScrollPane1;
    // End of variables declaration//GEN-END:variables

	@Override
	public void mouseClicked(MouseEvent e)
	{
		if( (_lastURL != null) && Desktop.isDesktopSupported() )
		{
			try
			{
				Desktop.getDesktop().browse( new URI( _lastURL ) );
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
		}
	}

	@Override
	public void mousePressed(MouseEvent e)
	{
	}

	@Override
	public void mouseReleased(MouseEvent e)
	{
	}

	@Override
	public void mouseEntered(MouseEvent e)
	{
	}

	@Override
	public void mouseExited(MouseEvent e)
	{
	}

	@Override
	public void mouseDragged(MouseEvent e)
	{
		moveMouse(e);
	}

	@Override
	public void mouseMoved(MouseEvent e)
	{
		moveMouse(e);
	}

	@Override
	public void mouseWheelMoved(MouseWheelEvent evt)
	{
		boolean pageHasBeenChanged = false;
//		int units = evt.getUnitsToScroll();
		int units = evt.getUnitsToScroll();

		Component parentComp = ((Component) evt.getSource()).getParent();
		JViewport jvp = null;
		if( parentComp instanceof JViewport )
		{
			jvp = (JViewport) parentComp;
		}

		JScrollPane scrollPane = null;
		if( jvp.getParent() instanceof JScrollPane )
		{
			scrollPane = (JScrollPane) jvp.getParent();
		}

		if( scrollPane != null )
		{
			JScrollBar scrollBar = null;
			scrollBar = scrollPane.getVerticalScrollBar();

			incrementScrollBarValue( scrollBar, (int) ( Math.signum(units) * 1 / 3 * scrollBar.getBlockIncrement(1) ) );

/*
			if (evt.getScrollType() == MouseWheelEvent.WHEEL_UNIT_SCROLL)
			{
				incrementScrollBarValue( scrollBar, scrollBar.getBlockIncrement(1) );
//				incrementScrollBarValue( scrollBar, evt.getUnitsToScroll() * scrollBar.getUnitIncrement() );
//						incrementScrollBarValue( scrollBar, ( units * scrollBar.getVisibleAmount() ) / 12 );
			}
			else
			{ //scroll type == MouseWheelEvent.WHEEL_BLOCK_SCROLL
				incrementScrollBarValue( scrollBar, scrollBar.getBlockIncrement(1) );
//				incrementScrollBarValue( scrollBar, scrollBar.getBlockIncrement(1) );
//						incrementScrollBarValue( scrollBar, scrollBar.getVisibleAmount() / 12 );
			}
*/
		}

		moveMouse(evt);
	}

	protected void incrementScrollBarValue( JScrollBar scrollBar, int increment )
	{
		setScrollBarValue( scrollBar, scrollBar.getValue() + increment );
	}

	protected void setScrollBarValue( JScrollBar scrollBar, int value )
	{
		int valueToSet = IntegerFunctions.min( scrollBar.getMaximum(),
												IntegerFunctions.max( scrollBar.getMinimum(), value) );

//		_scrollByProgram = true;
		scrollBar.setValue(valueToSet);
//		_scrollByProgram = false;
	}

	public void moveMouse( MouseEvent e )
	{
		Component parent = ((Component) e.getSource()).getParent();
		moveMouse( parent );
	}

	protected void moveMouse( Component parent )
	{
		if( parent instanceof JViewport )
		{
			JViewport jvp = (JViewport) parent;
			changeUrlCursor( jvp, -1 );
		}
	}
	
	protected void changeUrlCursor( JViewport textComponentParent, int oldScrollYYValue )
	{
		if( textComponentParent.getComponent(0) instanceof JTextComponent )
		{
			JTextComponent jtc = (JTextComponent) textComponentParent.getComponent(0);

			Point pt = MouseFunctions.getMouseLocation();
			if( ScreenFunctions.isInsideComponent( textComponentParent, pt ) )
			{
				Point parentPosition = textComponentParent.getLocationOnScreen();

				if( textComponentParent.getParent() instanceof JScrollPane )
				{
					JScrollPane parent = (JScrollPane) textComponentParent.getParent();

					int scrollYYValue = parent.getVerticalScrollBar().getValue();
					
					pt = new Point( (int) Math.min( jtc.getWidth()-1,
													(pt.getX() - parentPosition.getX()) ),
									(int) Math.min( jtc.getHeight()-1,
													(double) (  ( pt.getY() - parentPosition.getY() ) *
															 parent.getVerticalScrollBar().getVisibleAmount()) /
														textComponentParent.getHeight() ) +
														scrollYYValue );
					int pos = jtc.viewToModel(pt);

					_lastURL = getCurrentUrl( jtc, pos );

//					System.out.println( "pt:" + pt + " position: " + pos + "  url:" + _lastURL );

					boolean cursorChanged = false;
					if( ( _lastURL != null ) && !jtc.getCursor().getName().equals( AvailableCursors._handCursor.getName() ) )
					{
						jScrollPane1.getViewport().getComponent(0).setCursor( AvailableCursors._handCursor );
						cursorChanged = true;
					}
					else if( (_lastURL == null ) && !jtc.getCursor().getName().equals( AvailableCursors._defaultCursor.getName() ) )
					{
						jScrollPane1.getViewport().getComponent(0).setCursor( AvailableCursors._defaultCursor );
						cursorChanged = true;
					}
					
					if( cursorChanged && ( oldScrollYYValue != -1 ) &&
						( oldScrollYYValue != scrollYYValue ) )
					{
						jScrollPane1.getVerticalScrollBar().setValue( oldScrollYYValue );
						jScrollPane1.getVerticalScrollBar().setValue( scrollYYValue );
/*						java.awt.EventQueue.invokeLater(new Runnable() {
							public void run() {
								_lensJPanel.repaint();
							}
						});
	*/
					}
				}
			}
		}
	}

	public void changeUrlCursor( int oldScrollYYValue )
	{
		changeUrlCursor(jScrollPane1.getViewport(), oldScrollYYValue );
	}

	protected String getCurrentUrl( JTextComponent tc, int pos )
	{
		String result = null;

		String text = null;

		try
		{
			text = tc.getDocument().getText( 0, tc.getDocument().getLength() );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			text = "";
		}

		int initialPosition = StringFunctions.lastIndexOfAnyChar( text, " \t\r\n\r", pos );
		initialPosition++;

		int finalPosition = StringFunctions.indexOfAnyChar( text, " \t\r\n\r", pos );
		if( finalPosition == -1 )
			finalPosition = text.length();

		if( (finalPosition - initialPosition) > 4 )
		{
			String word = text.substring( initialPosition, finalPosition );

			if( ( word.length() > 4 ) && word.substring( 0, 4 ).equals( "http" ) )
				result = word;
		}

		return( result );
	}
}
