/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.textcrypt.controller.threads;

import com.frojasg1.general.desktop.TextWrapper;
import com.frojasg1.general.progress.CancellationException;
import com.frojasg1.general.progress.OperationCancellation;
import com.frojasg1.general.progress.UpdatingProgress;
import com.frojasg1.textcrypt.controller.TextcryptController;
import com.hotmail.frojasg1.applications.fileencoder.FileEncoder;
import com.hotmail.frojasg1.applications.fileencoder.FileEncoderException;
import com.hotmail.frojasg1.applications.fileencoder.FileEncoderParameters;
import com.hotmail.frojasg1.applications.fileencoder.fileencodertypes.FileEncoderType_3;
import java.io.File;
import java.nio.file.Files;

/**
 *
 * @author axpe
 */
public class OpenEncryptedTextThread extends ParentThread
{
	protected String _fileName = null;
	protected char[] _password = null;
	
	public OpenEncryptedTextThread( TextcryptController controller,
									String fileName,
									char[] password,
									OperationCancellation oc )
	{
		super( controller, oc );
		_fileName = fileName;
		_password =  password;
	}

	@Override
    public void run()
    {
		String text = null;
		int strength = 256;

		try
		{
			FileEncoderParameters fep = new FileEncoderParameters();
			fep.M_setFileEncoderType( FileEncoderParameters.INT_FILE_ENCODER_TYPE_4 );

			byte[] data = FileEncoder.M_decodeFileToBuffer( _password, fep, _fileName, _controller.getView(), a_oc );
			if( data != null )
				text = new String( data, "UTF8" );
			else
				text = "";
			
			strength = fep.M_getSizeOfNumbersSimpleEncoder();
		}
		catch( FileEncoderException ex )
		{
			M_setErrorString( ex.getMessage() );
		}
		catch (CancellationException ex)
		{
			M_setIsCancellation( true );
		}
		catch( Throwable th )
		{
			M_setErrorString( th.getMessage() );
		}

		if( M_getItWasError() )
        {
            _controller.textCrypt_fileError( M_getErrorString() );
        }
		else if( M_isCancellation() )
		{
				_controller.textCrypt_fileCancelled();
		}
		else
			_controller.textCrypt_fileOpen( text, strength );

        M_setHasEnded(true);
    }
}
